package net.wanji.opt.servicev2.implv2;

import lombok.extern.slf4j.Slf4j;
import net.wanji.common.enums.BaseEnum;
import net.wanji.common.utils.tool.StringUtils;
import net.wanji.opt.dao.mapper.CrossMapper;
import net.wanji.opt.dao.mapper.GreenwaveHistoryMapper;
import net.wanji.opt.po.base.CrossLaneDataHistPoExtend;
import net.wanji.opt.servicev2.CrossService;
import net.wanji.opt.synthesis.pojo.*;
import net.wanji.opt.vo2.CrossBaseInfoVO;
import net.wanji.opt.vo2.CrossStatusDistributionVO;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 监测详情-路口事件详情
 *
 * @author zhengyifan
 * @date 2025/3/15
 */
@Service
@Slf4j
public class CrossServiceImpl implements CrossService {
    @Resource
    private CrossMapper crossMapper;
    @Resource
    private GreenwaveHistoryMapper greenwaveHistoryMapper;

    /**
     * 路口状态分布
     *
     * @param crossID       路口ID
     * @param date          日期
     * @param groupType     时间粒度 参数要求：0--5分钟 1--15分钟 2--30分钟 3--60分钟
     * @param objectType    范围 参数要求：1--方向级 2--转向级 3--车道级 4--路口级
     * @param directionName 方向 东南西北
     * @return
     */
    @Override
    public List<CrossStatusDistributionVO> getCrossStatusDistribution(String crossID, String date, String groupType, Integer objectType, String directionName) {
        Map<String, Object> params = new HashMap<>();
        params.put("crossId", crossID);
        params.put("startDate", date + " 00:00:00");
        params.put("endDate", date + " 23:59:59");
        params.put("groupType", groupType);
        params.put("objectType", objectType);
        if (StringUtils.isNotBlank(directionName)) {
            params.put("dir", BaseEnum.SignalDirectionEnum.getCodeByName(directionName.substring(0, directionName.indexOf("进口"))));
        }

        List<CrossLaneDataHistPoExtend> firstList = greenwaveHistoryMapper.findCrossObjectIndex(params);
        List<CrossStatusDistributionVO> results = new ArrayList<>();

        for (CrossLaneDataHistPoExtend c: firstList) {
            CrossStatusDistributionVO cd = new CrossStatusDistributionVO();

            cd.setSpeed(c.getSpeed());
            cd.setStartTime(c.getStartTime().toString().substring(11, 16));
            cd.setDelayTime(c.getDelayTime());
            cd.setTurnType(c.getTurnType());
            cd.setTurnTypeName(BaseEnum.TurnTypeEnum.getNameByCode(c.getTurnType()));
            cd.setDirType(c.getDirType());
            cd.setDirTypeName(BaseEnum.SignalDirectionEnum.getNameByCode(c.getDirType()));
            cd.setStopTimes(c.getStopTimes());
            cd.setQueueLength(c.getAvgQueueLength());
            cd.setLaneNo(c.getLaneNo());

            results.add(cd);

        }

        return results;
    }

    /**
     * 路口实时告警
     *
     * @param crossID 路口ID
     * @param time    可选参数
     * @return
     */
    @Override
    public List<CrossRealTimeAlarmEntity> getCrossRealTimeAlarm(String crossID, String time) {
        return crossMapper.getCrossRealTimeAlarms(crossID, time);
    }

    @Override
    public List<CrossStatusDisOptTimeEntity> getOptTimeList(String crossID) {
        return crossMapper.getOptTimes(crossID);
    }

    /**
     * 路口基础信息————方向、转向、车道信息
     *
     * @param crossID 路口ID
     */
    @Override
    public CrossBaseInfoVO getCrossBaseInfos(String crossID) {
        List<CrossBaseInfoVO> crossBaseInfoVOS = new ArrayList<>();

        List<CrossDirectionInfoEntity> directionInfoEntities = crossMapper.getCrossDirectionInfo(crossID);
        List<CrossTurnInfoEntity> turnInfoEntities = crossMapper.getCrossTurnInfo(crossID);
        List<CrossLaneInfoEntity> laneInfoEntities = crossMapper.getCrossLaneInfo(crossID);

        CrossBaseInfoVO crossBaseInfoVO = new CrossBaseInfoVO();
        crossBaseInfoVO.setCrossID(crossID);
        crossBaseInfoVO.setDirectionInfo(directionInfoEntities);
        crossBaseInfoVO.setTurnInfo(turnInfoEntities);
        crossBaseInfoVO.setLaneInfo(laneInfoEntities);

        return crossBaseInfoVO;
    }
}
